// file: $JEIDA/src/check_channels/v1.0/src/check_channels.cc
//
// this is a simple program that checks whether each channel of the
// data appears to be the same size.
//

// system include files
//
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

// isip include files
//
#include <math/integral/integral.h>
#include <math/integral/integral_constants.h>

// important constants
//
#define MAX_NAME_LENGTH 999
#define MAX_MAG 512

// main program
//
main (int argc, char** argv) {

  // check the arguments
  //
  if (argc != 2) {
    fprintf(stdout, "usage: %s file_list\n", argv[0]);
    return(ISIP_ERROR);
  }

  // variables for filenames
  //
  char_1 buf[MAX_NAME_LENGTH];
  char_1 buf_c1[MAX_NAME_LENGTH];

  // fetch all of the filenames
  //
  FILE* fp_l = fopen(argv[1], "r");
  int_4 counter = (int_4)0;
  int_4 size_c0 = -1;
  int_4 size_c1 = -1;

  while(fgets((char*)buf, MAX_NAME_LENGTH-1, fp_l) != NULL) {

    // open the file
    //
    buf[strlen((char*)buf) - 1] = '\0';

    FILE* fp_f = fopen((char*)buf, "r");
    if (fp_f == NULL) {
      fprintf(stdout, "%ld: %s doesn't exist\n", counter, buf);
      fclose(fp_f);
      size_c0 = -1;
    }

    else {

      // compute the size of the file
      //
      fseek(fp_f, 0, SEEK_SET); int_4 beg = ftell(fp_f);
      fseek(fp_f, 0, SEEK_END); size_c0 = ftell(fp_f) - beg;
      fseek(fp_f, 0, SEEK_SET);

      // check the max ampl (a simple check for zero-valued data)
      //
      int_4 num_samples = size_c0 / sizeof(int_2);
      int_2* sig = new int_2[num_samples];
      if ((int_4)fread(sig, sizeof(int_2), num_samples, fp_f) < num_samples)
	{printf("***> error reading file %s\n", buf);}

      int_4 max_mag = 0;
      for (int_4 n=0; n<num_samples; n++) {
	if (sig[n] > max_mag)
	  {max_mag = sig[n];}
	else if (-sig[n] > max_mag)
	  {max_mag = -sig[n];}
      }

      delete [] sig;
      if (max_mag < MAX_MAG) {
	fprintf(stdout, "file %s\n", buf);
	fprintf(stdout, " max_mag is too small [%ld < %ld]\n",
		(long int)max_mag, (long int)MAX_MAG);
      }

      // close the file
      //
      fclose(fp_f);

    }

    // create the new filename
    //
    strcpy((char*)buf_c1, (char*)buf);
    char_1* str_chan = (char_1*)strstr((char*)buf_c1, "c0");
    *str_chan = 'c'; *(str_chan+1) = '1';

    fp_f = fopen((char*)buf_c1, "r");
    if (fp_f == NULL) {
      fprintf(stdout, "%ld: %s doesn't exist\n", counter, buf_c1);
      fclose(fp_f);
      size_c1 = -1;
    }

    else {

      // compute the size of the file
      //
      fseek(fp_f, 0, SEEK_SET); int_4 beg = ftell(fp_f);
      fseek(fp_f, 0, SEEK_END); size_c1 = ftell(fp_f) - beg;
      fseek(fp_f, 0, SEEK_SET);

      // check the max ampl (a simple check for zero-valued data)
      //
      int_4 num_samples = size_c0 / sizeof(int_2);
      int_2* sig = new int_2[num_samples];
      if ((int_4)fread(sig, sizeof(int_2), num_samples, fp_f) < num_samples)
	{printf("***> error reading file %s\n", buf);}

      int_4 max_mag = 0;
      for (int_4 n=0; n<num_samples; n++) {
	if (sig[n] > max_mag)
	  {max_mag = sig[n];}
	else if (-sig[n] > max_mag)
	  {max_mag = -sig[n];}
      }

      delete [] sig;
      if (max_mag < MAX_MAG) {
	fprintf(stdout, "file %s\n", buf_c1);
	fprintf(stdout, " max_mag is too small [%ld < %ld]\n",
		(long int)max_mag, (long int)MAX_MAG);
      }

      // close the file
      //
      fclose(fp_f);
    }

    // check the sizes
    //
    if (size_c0 != size_c1) {
      fprintf(stdout, "file %s\n", buf);
      fprintf(stdout, " and file %s\n", buf_c1);
      fprintf(stdout, " are not the same sizes [%ld to %ld]\n", size_c0, size_c1);
    }

    // increment the counter
    //
    counter++;
    if (counter % 100 == 0)
      {fprintf(stdout, "... processing file %ld...\n", counter);}
  }

  // close the file
  //
  fclose(fp_l);

  // exit gracefully
  //
  fprintf(stdout, "... done - processed %ld files ...\n", counter);
  return(ISIP_NO_ERROR);
}
