// file: $JEIDA/src/check_files/v1.0/src/check_files.cc
//
// this is a simple program that checks the uniqueness of files.
// the check is simple:
//
//  (a) are the files the same size?
//  (b) are the first few bytes the same?
//
// it also checks how long the files are. If they are longer than some
// preset threshold, a warning is issued.
//
// note that this program does an NxN comparison of files.
//

// system include files
//
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

// isip include files
//
#include <math/integral/integral.h>
#include <math/integral/integral_constants.h>

// important constants
//
#define MAX_NUM_FNAMES 499999
#define MAX_NAME_LENGTH 999
#define CHECK_SIZE 256

#define DIFFERENT 0
#define SAME_DATA 1
#define NOM_SIZE_IN_BYTES 50000

// local functions
//
int_4 compare_files(char_1* buf_1, char_1* buf_2);


// main program
//
main (int argc, char** argv) {

  // check the arguments
  //
  int_4 nom_size_in_bytes = NOM_SIZE_IN_BYTES;

  if (argc == 3) {
    nom_size_in_bytes = atoi(argv[2]);
  }
  else if (argc != 2) {
    fprintf(stdout, "usage: %s file_list\n", argv[0]);
    return(ISIP_ERROR);
  }

  // variables for filenames
  //
  char_1* fnames[MAX_NUM_FNAMES];
  char_1* data[MAX_NUM_FNAMES];
  int_4 dsizes[MAX_NUM_FNAMES];
  char_1 buf[MAX_NAME_LENGTH];

  // fetch all of the filenames, and the buffers associated with each
  //
  FILE* fp_l = fopen(argv[1], "r");
  int_4 counter = (int_4)0;

  while(fgets((char*)buf, MAX_NAME_LENGTH-1, fp_l) != NULL) {

    // save the filename
    //
    buf[strlen((char*)buf)-1] = '\0';
    fnames[counter] = new char_1[strlen((char*)buf) + 1];
    strcpy((char*)fnames[counter], (char*)buf);

    // open the file
    //
    FILE* fp_f = fopen((char*)fnames[counter], "r");
    if (fp_f == NULL) {
      fprintf(stdout, "%ld: %s doesn't exist\n", counter, fnames[counter]);
      fclose(fp_f);
    }
    else {

      // compute the size of the file
      //
      fseek(fp_f, 0, SEEK_SET); int_4 beg = ftell(fp_f);
      fseek(fp_f, 0, SEEK_END); dsizes[counter] = ftell(fp_f) - beg;
      fseek(fp_f, 0, SEEK_SET);

      if (dsizes[counter] > nom_size_in_bytes) {
	fprintf(stdout, "%ld: %s\n", counter, fnames[counter]);
	fprintf(stdout, "        <warning> file too large %ld\n", dsizes[counter]);
      }

      // allocate space
      //
      data[counter] = (char_1*)malloc(CHECK_SIZE);

      // read the data
      //
      if (fread(data[counter], CHECK_SIZE, (int)1, fp_f) != (int)1) {
	fprintf(stdout, "%ld: %s\n", counter, fnames[counter]);
	fprintf(stdout, "        **> error reading the data\n");
      }

      // close the file
      //
      fclose(fp_f);

      // increment the counter
      //
      counter++;
      if (counter % 100 == 0)
	{fprintf(stdout, "... processing file %ld...\n", counter);}
    }
  }

  // close the file
  //
  fclose(fp_l);

  int_4 num_fnames = counter;
  fprintf(stdout, "... done reading %ld files ...\n", num_fnames);

  // loop over all files and check
  //
  fprintf(stdout, "... checking files ...\n");

  for (int_4 i=0; i<num_fnames; i++) {

    if (i % 100 == 0) {
      fprintf(stdout, " >>> checking file no. %ld\n", i);
    }

    for (int_4 j=(i+1); j<num_fnames; j++) {

      // check the files
      //
      int_4 flag_check;

      if (dsizes[i] != dsizes[j])
	{flag_check = DIFFERENT;}
      else
	{flag_check = compare_files(data[i], data[j]);}

      if (flag_check == SAME_DATA) {
	fprintf(stdout, "(%ld,%ld) ***> error:\n", i, j);
	fprintf(stdout, "                      file: %s\n", fnames[i]);
	fprintf(stdout, "                  and file: %s\n", fnames[j]);
	fprintf(stdout, "                      are the same!\n");
      }
    }
  }

  // exit gracefully
  //
  return(ISIP_NO_ERROR);
}


// function: compare_files
//
int_4 compare_files(char_1* buf_1, char_1* buf_2) {

  // check the first chunk of data
  //
  logical_1 flag_diff = ISIP_FALSE;

  for (int_4 i=0; i<CHECK_SIZE; i++) {
    if (buf_1[i] != buf_2[i]) {
      flag_diff = ISIP_TRUE;
      break;
    }
  }

  // return the proper value
  //
  if (flag_diff == ISIP_TRUE)
    {return DIFFERENT;}
  else
    {return SAME_DATA;}
}

