// file: $JEIDA/src/make_files/v2.0/src/make_files.cc
//
// this program operates on raw endpointed data and 
// splits it into files/directories
//
// usage:
//		make_files foo.raw foo.endpts
//
// action:
//		breaks foo.raw into files of the form
//			c_0/set???/f####_[0,1].raw
//		

// system include files
//
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <libgen.h>

// isip include files
//
#include <math/integral/integral.h>
#include <math/integral/integral_constants.h>

// important constants
//
#define MAX_STRING_LENGTH 999
#define ENDPT_FORMAT "%ld%*s%*s%f%*s%*s%*s%f%*s"

#define DIR_PROT 0775
#define DIR_FORMAT "c_%1ld/set_%2.2ld"
#define FILE_FORMAT "c_%1ld/set_%2.2ld/f%4.4ld_%1ld.raw"

#define NUM_FILES_PER_DIR 100
#define SAMPLE_FREQ 16000
#define SAMPLE_SIZE 2
#define NUM_CHAN 2

// main program
//
main (int argc, char** argv) {

  // check the arguments
  //
  if (argc != 3) {
    fprintf(stdout, "usage: %s foo.raw foo.endpts\n", argv[0]);
    return(ISIP_ERROR);
  }

  // variables for filenames
  //
  char_1* file_raw = (char_1*)argv[1];
  char_1* file_end = (char_1*)argv[2];
  char_1  dir_name[MAX_STRING_LENGTH];
  char_1  full_name_0[MAX_STRING_LENGTH];
  char_1  full_name_1[MAX_STRING_LENGTH];

  // open the input files
  //
  FILE* fp_r = fopen((char*)file_raw, "r");
  if (fp_r == NULL) {
    fprintf(stdout, "*> error opening %s\n", file_raw);
    return(ISIP_ERROR);
  }
  
  FILE* fp_e = fopen((char*)file_end, "r");
  if (fp_e == NULL) {
    fprintf(stdout, "*> error opening %s\n", file_end);
    return(ISIP_ERROR);
  }
  
  // loop over all endpoints
  //
  int_4 utt_num;
  float_4 beg_time, end_time;

  while (fscanf(fp_e, ENDPT_FORMAT, &utt_num, &beg_time, &end_time) >= 0) {

    int_4 dir_num = utt_num/NUM_FILES_PER_DIR;

    // check if we need to make a directory
    //
    if (utt_num % NUM_FILES_PER_DIR == 0) {

      // make channel 0
      //
      sprintf((char*)dir_name, DIR_FORMAT, (int_4)0, dir_num);
      if (mkdirp((char*)dir_name, DIR_PROT) < (int)0) {
	fprintf(stdout, "cannot create directory %s\n", dir_name);
	return(ISIP_ERROR);
      }

      // make channel 1
      //
      sprintf((char*)dir_name, DIR_FORMAT, (int_4)1, dir_num);
      if (mkdirp((char*)dir_name, DIR_PROT) < (int)0) {
	fprintf(stdout, "cannot create directory %s\n", dir_name);
	return(ISIP_ERROR);
      }
    }

    // open the output files
    //
    sprintf((char*)full_name_0, FILE_FORMAT,
	    (int_4)0, dir_num, utt_num, (int_4)0);
    fprintf(stdout, "\t...creating file %s\n", full_name_0);

    FILE* fp_0 = fopen((char*)full_name_0, "w");
    if (fp_0 == NULL) {
      fprintf(stdout, "*> error opening %s\n", full_name_0);
      return(ISIP_ERROR);
    }

    sprintf((char*)full_name_1, FILE_FORMAT,
	    (int_4)1, dir_num, utt_num, (int_4)1);
    FILE* fp_1 = fopen((char*)full_name_1, "w");
    if (fp_1 == NULL) {
      fprintf(stdout, "*> error opening %s\n", full_name_1);
      return(ISIP_ERROR);
    }

    // convert times to samples
    //
    int_4 offset = (int_4)(beg_time * SAMPLE_FREQ * SAMPLE_SIZE * NUM_CHAN);
    int_4 num_samples = (int_4)((end_time - beg_time) * SAMPLE_FREQ);
    if ((offset < (int_4)0) || (num_samples < (int_4)0)) {
      fprintf(stdout, "*> incorrect endpts for file %s (%ld)\n",
	      full_name_0, num_samples);
      return(ISIP_ERROR);
    }

    // create space
    //
    int_2* sig_s = new int_2[num_samples*NUM_CHAN];
    int_2* sig_0 = new int_2[num_samples];
    int_2* sig_1 = new int_2[num_samples];

    // read data
    //
    fseek(fp_r, offset, SEEK_SET);
    if (fread(sig_s, NUM_CHAN * SAMPLE_SIZE,
	      num_samples, fp_r) < (uint_4)num_samples) {
      fprintf(stdout, "*> error reading data from %s\n", full_name_0);
      return(ISIP_ERROR);
    }

    // transfer data
    //
    for (int_4 n=0; n<num_samples; n++) {
      sig_0[n] = sig_s[n*NUM_CHAN];
      sig_1[n] = sig_s[n*NUM_CHAN+1];
    }    

    // write data
    //
    if (fwrite(sig_0, SAMPLE_SIZE, num_samples, fp_0) < (uint_4)num_samples) {
      fprintf(stdout, "*> error writing data to %s\n", full_name_0);
      return(ISIP_ERROR);
    }

    if (fwrite(sig_1, SAMPLE_SIZE, num_samples, fp_1) < (uint_4)num_samples) {
      fprintf(stdout, "*> error writing data to %s\n", full_name_1);
      return(ISIP_ERROR);
    }

    // close up
    //
    fclose(fp_0);
    fclose(fp_1);

    // clean up
    //
    delete [] sig_s;
    delete [] sig_0;
    delete [] sig_1;

  }

  // close the file
  //
  fclose(fp_r);
  fclose(fp_e);

  // exit gracefully
  //
  fprintf(stdout, "... done ... processed %ld files ...\n", utt_num+1);
  return(ISIP_NO_ERROR);
}
