
/* File: main.c */
   
/*
 * Richard Duncan
 * Institute for Signal and Information Processing
 * December 1995
 *
 *
 * This program converts a list of raw speech data files and JEIDA validation
 * files into the NIST sphere format data files
 *
 */
 
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#ifndef __SP_SPHERE_H_
#define __SP_SPHERE_H_
#include <sp/sphere.h>
#endif

#ifndef __MAIN_H_
#define __MAIN_H_
#include "main.h"
#endif

#ifndef __CREATE_SPHERE_FILE_H_
#define __CREATE_SPHERE_FILE_H_
#include "create_sphere_file.h"
#endif


int main(int argc, char **argv){
  
  char session_info_filename[MAX_FILENAME_SIZE];
  char val_file_list_filename[MAX_FILENAME_SIZE];
  char raw_data_file_list_filename[MAX_FILENAME_SIZE];

  char val_filename[MAX_FILENAME_SIZE];
  char raw_data_filename[MAX_FILENAME_SIZE];
  char sphere_filename[MAX_FILENAME_SIZE];

  char com_str[MAX_FILENAME_SIZE];
  
  FILE *session_info_fp;
  FILE *val_file_list_fp;
  FILE *raw_data_file_list_fp;

  int n = 0;
  char prog_name[MAX_FILENAME_SIZE];

  char *temp;

  if(strchr(*argv, '/')){
    strcpy(prog_name, strrchr(*argv, '/')+1);
    printf(info, prog_name, "1.0");
  }else{
    strcpy(prog_name, *argv);
  }
  for(n=0;n<argc;n++){
    if(strstr(argv[n], "-help")){
      printf("name: %s\n", prog_name);
      printf("synopsis: %s <session_file> <val_file_list> <raw_file_list>\n", prog_name);
      printf("example: %s SESSION_INFO_FILE.text val_list.text raw_list.text destination\n", prog_name);
      printf("\noptions:\n");
      printf("  -help: display this help message\n");
      printf("\narguments: file names\n");
      printf("  session_file: file of session information in field = value format\n");
      printf("  val_file_list: file containing a list of JEIDA validation files\n");
      printf("  raw_file_list: file containing a list of RAW format audio files\n");
      printf("  destination  : a directory in which to place the sphere files\n");
      printf("  *note: every line of val_file_list should correspond to raw_file_list\n");
      printf("\nman page: none\n");
      exit(0);
    }
  }
  if (argc == 5){
    strcpy(session_info_filename, argv[1]);
    strcpy(val_file_list_filename, argv[2]);
    strcpy(raw_data_file_list_filename, argv[3]);

    if ((session_info_fp = fopen(session_info_filename,"r")) == NULL) {
      fprintf(stderr, file_error, "session info", session_info_filename);
      exit (10);
    }
    if ((val_file_list_fp = fopen(val_file_list_filename,
				  "r")) == NULL) {
      fprintf(stderr, file_error, "val file list",
	      val_file_list_filename);
      exit(11);
    }
    if ((raw_data_file_list_fp = fopen(raw_data_file_list_filename,
				       "r")) == NULL ) {
      fprintf(stderr, file_error, "raw data file list",
	      raw_data_file_list_filename);
      exit(12);
    }
    printf("Using %s\n", sp_get_version()); 
    while ((!feof(val_file_list_fp))&&
	   (!feof(raw_data_file_list_fp))) {
      
      /*
       * read in the filenames from the lists
       */
      
      fscanf(val_file_list_fp, "%s\n", val_filename);
      fscanf(raw_data_file_list_fp, "%s\n", raw_data_filename);
      
      /*
       * create the appropriate sphere filename from the raw filename
       *
       */

      strcpy(sphere_filename,argv[4]);

      if(sphere_filename[strlen(sphere_filename) -1] == '/'){
	sphere_filename[strlen(sphere_filename) - 1] = (char)NULL;
      }

      if (strchr(raw_data_filename,(int)'/')){
	temp = strrchr(raw_data_filename,(int)'/');
      }
      else {
	temp = raw_data_filename;
	strcat(sphere_filename,"/");
      }
      
      strcpy(&sphere_filename[strlen(sphere_filename)], temp);
      strcpy(strstr(sphere_filename, raw_file_extension),
	     sphere_file_extension);
      
      printf("\nConverting %s --> %s\n",
	      raw_data_filename, sphere_filename);
      /*
       * create the sphere file
       *
       */
      create_sphere_file(raw_data_filename, sphere_filename, val_filename,
			 session_info_fp);
      
    }
    /*
     * Make sure that both the files are at EOF, not just one
     *
     */
    if ((!feof(val_file_list_fp)) ||
	(!feof(raw_data_file_list_fp))) {
      fprintf(stderr, size_error);
      exit(20);
    }
    fclose(val_file_list_fp);
    fclose(raw_data_file_list_fp);
    printf("\nEnding normally\n");
  } else {
    fprintf(stderr,usage,prog_name, prog_name);
  }
}  

  



