/* File: read_val_file.c */

/*
 * Richard Duncan
 * Institute for Signal and Information Processing
 * December 1995
 *
 *
 * This function reads in the neccessary fields from the valfile into
 * the header, and converts the transcription into the prompting text
 * for the header
 *
 */

#include<stdio.h>

#ifndef __SP_SPHERE_H_
#define __SP_SPHERE_H_
#include <sp/sphere.h>
#endif

#ifndef __MAIN_H_
#define __MAIN_H_
#include "main.h"
#endif

#ifndef __GET_PROMPTING_TEXT_H_
#define __GET_PROMPTING_TEXT_H_
#include "get_prompting_text.h"
#endif

#ifndef __READ_VAL_FILE_H_
#define __READ_VAL_FILE_H_
#include "read_val_file.h"
#endif

int read_val_file(struct header_t *header,
		  char *val_filename){

  FILE *val_fp;
  char field_value[128];
  char field_name[64];
  SP_REAL float_value;
  SP_INTEGER integer_value;
  
  /*
   * open the validation file
   *
   */
  
  if ((val_fp = fopen(val_filename, "r")) == NULL) {
    fprintf(stderr, file_error, "val file", val_filename);
    exit(50);
  }

  /*
   * read in the sample_frequency and write it to sample_rate field
   *
   */
  
  rewind(val_fp);
  while((!feof(val_fp)) && strcmp(field_name, "sample_frequency")){
    fscanf(val_fp, "%s = %s", field_name, field_value);
  }
  integer_value = (SP_INTEGER) atoi(field_value);
  if(integer_value > 0){
    if(sp_add_field(header, "sample_rate", T_INTEGER,
		    (char *)&integer_value) < 0){
      fprintf(stderr, field_error, "sample_rate");
      exit(53);
    }
  } else {
    fprintf(stderr, field_read_error, "sample_frequency");
    exit(54);
  }

  /*
   * read in the transcription
   *
   */

  rewind(val_fp);
  while((!feof(val_fp)) && strcmp(field_name, "transcription")){
    fscanf(val_fp, "%s = \"%[^\"]\"", field_name, field_value);
  }
  if(!feof(val_fp)){
    if(sp_add_field(header, "orthographic_transcription",
		    T_STRING, field_value) < 0){
      fprintf(stderr, field_error, "orthographic_transcription");
      exit(53);
    }
  } else {
    fprintf(stderr, field_read_error, "transcription");
    exit(54);
  }

  printf("\ttranslating '%s' -> ", field_value);
  
  /*
   * calculate the prompting text from the transcription
   *
   */

  get_prompting_text(field_value, field_value);
  if(sp_add_field(header, "prompting_text", T_STRING, field_value) < 0){
    fprintf(stderr, field_error, "prompting_text");
    exit(53);
  }
  printf("'%s'\n",field_value);

  /*
   * close the validation file
   *
   */
  
  fclose(val_fp);


}



