 // file: endpoint_calculator.cc
 
#include "endpointer.h"

// external functions prototypes

void energy_2(FILE *, int, float *, int, struct parameter_data *);
void noise_2(float *, int, float *, float *, struct parameter_data *);
void signal_2(float *, int, float *, struct parameter_data *);
int find_critical_points(float *, float *, int, struct critical_points_struct *, int, struct parameter_data *);

// main function

float endpoint_calculator(FILE *source_file, FILE *output_file, struct parameter_data *parameter)
{
  
  // variable declarations

  int file_length;
  int samples_in_frame;
  float *energy_array, *noise_array, *signal_array;
  int number_of_frames;
  int counter;
  float maybe_utterance_start;
  float utterance_start;
  float utterance_end;
  float maybe_utterance_end;
  int utterance_number;
  int number_of_utterances;
  int number_of_critical_points;
  struct critical_points_struct *critical_point;
  struct endpoints_struct *endpoint;
  
  // initialize some values
  
  samples_in_frame = (int)(parameter->frame_duration*parameter->sample_frequency);
  
  // get file_length
  
  rewind(source_file);
  fseek(source_file,0L,SEEK_END);
  file_length = ftell(source_file);
  rewind(source_file);
  
  
  // allocate space for everything;
  
  number_of_frames = (int)((file_length / (parameter->number_of_channels * SAMPLE_SIZE * samples_in_frame)+1));
  energy_array = (float *)malloc(sizeof(float) * number_of_frames);
  signal_array = (float *)malloc(sizeof(float) * number_of_frames);
  noise_array  = (float *)malloc(sizeof(float) * number_of_frames);
  critical_point = (struct critical_points_struct *)malloc(sizeof(struct critical_points_struct) * CRITICAL_POINTS_MAX);
  endpoint = (struct endpoints_struct *)malloc(sizeof(struct endpoints_struct) * ENDPOINTS_MAX);
  
  
  // calculate energy, noise, signal, and critical points
  
  energy_2(source_file, file_length, energy_array, number_of_frames, parameter);
  printf("Energy levels calculated\n");
  signal_2(energy_array, number_of_frames, signal_array, parameter);
  printf("Signal levels calculated\n");
  noise_2(energy_array, number_of_frames, noise_array, signal_array, parameter);
  printf("Noise levels calculated\n");
  
  // find critical points
  //
  number_of_critical_points = 
    find_critical_points(signal_array, noise_array, number_of_frames,
			 critical_point, CRITICAL_POINTS_MAX, parameter);
  printf("Critical points found\n");
  
  // debugging stuff - prints out critical points
     printf("number of critical points = %d\n", number_of_critical_points);
     counter = 0;
     while(counter < number_of_critical_points)
     {
     printf("%6.3f %f\n", (critical_point + counter)->time, 0.0);
     printf("%6.3f %f\n", (critical_point + counter)->time,
	    2000*((critical_point + counter)->positive_slope - 0.5));
     printf("%6.3f %f\n", (critical_point + counter)->time, 0.0);
     counter ++;
     }
  // end of debugging stuff
  
  // debugging stuff - prints out energy, noise, and signal
 
     counter = 0;
     while (counter < number_of_frames)
     {
     printf("%5.2f %5.2f %5.2f %5.2f\n", (float)counter * parameter->frame_duration, energy_array[counter], noise_array[counter], signal_array[counter]);
     counter ++; 
     }
    
  // end of debugging stuff
  
  
  utterance_start = 0;
  utterance_end  = 0;
  maybe_utterance_start = 0;
  maybe_utterance_end = 0;
  counter=0;
  utterance_number =0;
  
  while (counter <  number_of_critical_points)
    {
      //    printf("maybe_utterance_start: %5.2f %d\n",critical_point->time, critical_point->positive_slope);
      
      maybe_utterance_start = critical_point->time;
      while(((critical_point+2)->time - (critical_point+1)->time) < parameter->minimum_utterance_separation && (counter < number_of_critical_points))
	{
	  //	 printf("in loop critical point: %5.2f %d\n",critical_point->time, critical_point->positive_slope);
	  
	  critical_point += 2;
	  counter += 2;
	  if(counter +2 > number_of_critical_points)
	    {
	      //	    printf("break\n");
	      critical_point -= 2;
	      break;
	    }
	}
      maybe_utterance_end = (critical_point + 1)->time;
      
      //    printf("maybe_utterance_end: %5.2f %d\n",(critical_point + 1)->time, (critical_point + 1)->positive_slope);
      
      if((maybe_utterance_end - maybe_utterance_start) > parameter->minimum_utterance_duration)
	{
	  
	  // rejoice! utterance has been found!
	  
	  utterance_start = maybe_utterance_start;
	  utterance_end = maybe_utterance_end;
	  (endpoint + utterance_number)->start_time = utterance_start;
	  (endpoint + utterance_number)->end_time = utterance_end;
	  utterance_number ++;
	}
      counter +=2;
      critical_point +=2;
      
    }
  
  printf("Raw endpoints found\n");
  
  // break utterances midway between the each endpoint and the following
  // startpoint, unless this is too big
  //
  float etime, stime, delta, ntime;
  
  number_of_utterances = utterance_number;

  float* e1 = (float*)malloc(number_of_utterances*sizeof(float));
  float* e2 = (float*)malloc(number_of_utterances*sizeof(float));  

  for (int utt_num=0; utt_num<number_of_utterances; utt_num++) {
    e1[utt_num] = (endpoint + utt_num)->start_time; 
    e2[utt_num] = (endpoint + utt_num)->end_time; 
//    printf("raw endpts: %f to %f \n", e1[utt_num],e2[utt_num]); 
  }

  for (utt_num=1; utt_num<number_of_utterances; utt_num++) {

    // find a new start point
    //
    stime = e1[utt_num];
    etime = e2[utt_num - 1];
    delta = stime - etime;

    printf("endpts before: %f to %f\n", etime, stime);

    ntime = etime + (stime - etime) * 0.5;

    if ((stime - ntime) < 4*parameter->initial_padtime) {
      (endpoint + utt_num)->start_time = ntime;
      (endpoint + utt_num - 1)->end_time = ntime;
      printf("   new start time is %f secs\n", ntime);
    }
    else {
      (endpoint + utt_num)->start_time -= parameter->initial_padtime;
      (endpoint + utt_num - 1)->end_time += parameter->final_padtime;
    }
    printf(" **> endpts after: %f to %f\n",
	   (endpoint + utt_num-1)->end_time,
	   (endpoint + utt_num)->start_time);
  }
  
// deal with special cases - first utterance and last utterance
  
  endpoint->start_time -= parameter->initial_padtime;
  if(endpoint->start_time < 0 )
    {
//      fprintf(stderr, "Warning: file may start with an utterance in progress\n");
      endpoint->start_time = 0;
    }
  
  (endpoint + number_of_utterances - 1)->end_time += parameter->final_padtime;
  if((endpoint + number_of_utterances)->end_time > number_of_frames * parameter->frame_duration)
    {
//       fprintf(stderr, "Warning: file may end with an utterance in progress\n");
       (endpoint + number_of_utterances -1)->end_time = number_of_frames * parameter->frame_duration;
     }

  printf("Endpoints padded\n");
    
// output utterances to file
    
    utterance_number = 0;
  while(utterance_number < number_of_utterances)
    {
      fprintf(output_file, "%4d beg_time = %7.2f secs     end_time = %7.2f secs\n",utterance_number, (endpoint + utterance_number)->start_time, (endpoint + utterance_number)->end_time);
      utterance_number ++;
    }
  
  return(0);
  
}
