# file: $JEIDA/src/validate/v1.0/src/select_wordlist.tcl
#
# These procedures allows the user to select a different wordlist
#

proc select_wordlist {} {

    # set up the globals
    #
    global env
    global p
    global swp
    global wordlist_label
    global wordlist_directory

    # set the name for the new toplevel
    #
    set swp ".swp"

    # destroy the swp toplevel if it exists
    #
    catch {destroy $swp}
    
    # create the new toplevel
    #
    toplevel $swp
    wm title $swp "Select New Wordlist"

    # create the frames that will be used in packing
    #
    frame $swp.top
    frame $swp.body
    frame $swp.foot

    # create the entry widget and label
    #
    entry $swp.wordlist -textvariable tmp_wordlist -relief sunken\
	    -width 50
    label $swp.label -relief raised -text "Wordlist:" -height 2

    # set the input focus to the entry
    #
    focus $swp.wordlist

    # create the ok button
    #
    button $swp.ok -text "ok" -padx 5 -command {
	set full_wordlist $wordlist_directory$tmp_wordlist
	set tmp_wordlist $full_wordlist
	readlist $tmp_wordlist
	set read_word_list_flag 1
	update_window
	destroy $swp
    }

    # bind return to execute "ok" and select wordlist
    #
    bind $swp.wordlist <Return> {
	set full_wordlist $wordlist_directory$tmp_wordlist
	set tmp_wordlist $full_wordlist
	readlist $tmp_wordlist
	set read_word_list_flag 1
	update_window
	destroy $swp
    }

    # create the cancel button
    #
    button $swp.cancel -text "cancel" -padx 5 -command {
	destroy $swp
    }

    # bind escape to cancel
    #
    bind $swp.wordlist <Escape> {
	destroy $swp
    }

    # create the listbox to select the different wordlist from
    #
    listbox $swp.wordlists -yscrollcommand "$swp.scroll set"

    # create a scrollbar for the listbox
    #
    scrollbar $swp.scroll -command "$swp.wordlists yview"

    # bind selection in the listbox
    #
    
    bind $swp.wordlists <Double-Button-1> {
	set tmp_wordlist [selection get]
    }
    
    # read in the possible wordlists from doing a directory of the
    # 
    foreach i [lsort [glob "$env(JEIDA)$wordlist_directory/*.text"]] {
	$swp.wordlists insert end [exec basename $i]
    }
       
    # pack the frames into the window
    #
    pack $swp.top $swp.body $swp.foot -side top -fill both

    # pack the top frame
    #
    pack $swp.label $swp.wordlist -in $swp.top -side left \
	    -fill both -expand true

    # pack the body
    #
    pack $swp.scroll -in $swp.body -side right -fill y
    pack $swp.wordlists -in $swp.body -side left -expand true -fill both

    # pack the foot
    #
    pack $swp.cancel $swp.ok -in $swp.foot -side left -expand true \
	    -fill both
    
}









